package dev.shadowsoffire.apotheosis.adventure.loot;

import net.minecraft.world.level.storage.loot.entries.LootPoolEntryContainer;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonElement;
import com.google.gson.reflect.TypeToken;
import com.google.gson.JsonArray;
import net.minecraft.util.GsonHelper;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonObject;
import net.minecraft.world.level.storage.loot.Serializer;
import dev.shadowsoffire.apotheosis.adventure.AdventureModule;
import net.minecraft.world.entity.player.Player;
import net.minecraft.util.random.WeightedRandom;
import net.minecraft.util.random.WeightedEntry;
import java.util.function.Predicate;
import net.minecraft.world.level.ServerLevelAccessor;
import net.minecraft.world.level.storage.loot.LootContext;
import net.minecraft.world.item.ItemStack;
import java.util.function.Consumer;
import java.util.stream.Stream;
import java.util.function.Function;
import java.util.Objects;
import net.minecraft.world.level.storage.loot.functions.LootItemFunction;
import net.minecraft.world.level.storage.loot.predicates.LootItemCondition;
import net.minecraft.resources.ResourceLocation;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import java.util.List;
import javax.annotation.Nullable;
import net.minecraft.world.level.storage.loot.entries.LootPoolEntryType;
import net.minecraft.world.level.storage.loot.entries.LootPoolSingletonContainer;

public class AffixLootPoolEntry extends LootPoolSingletonContainer
{
    public static final LootPoolEntryType TYPE;
    @Nullable
    private final RarityClamp.Simple rarityLimit;
    private final List<DynamicHolder<AffixLootEntry>> entries;
    
    public AffixLootPoolEntry(@Nullable final RarityClamp.Simple rarityLimit, final List<ResourceLocation> entries, final int weight, final int quality, final LootItemCondition[] conditions, final LootItemFunction[] functions) {
        super(weight, quality, conditions, functions);
        this.rarityLimit = rarityLimit;
        final Stream<Object> stream = entries.stream();
        final AffixLootRegistry instance = AffixLootRegistry.INSTANCE;
        Objects.requireNonNull(instance);
        this.entries = stream.map((Function<? super Object, ?>)instance::holder).toList();
    }
    
    protected void m_6948_(final Consumer<ItemStack> list, final LootContext ctx) {
        ItemStack stack;
        if (this.entries.isEmpty()) {
            final Player player = GemLootPoolEntry.findPlayer(ctx);
            if (player == null) {
                return;
            }
            final LootRarity selectedRarity = LootRarity.random(ctx.m_230907_(), ctx.m_78945_(), this.rarityLimit);
            stack = LootController.createRandomLootItem(ctx.m_230907_(), selectedRarity, player, (ServerLevelAccessor)ctx.m_78952_());
        }
        else {
            final List<WeightedEntry.Wrapper<AffixLootEntry>> resolved = this.entries.stream().map((Function<? super Object, ?>)this::unwrap).filter(Objects::nonNull).map(e -> e.wrap(ctx.m_78945_())).toList();
            final AffixLootEntry entry = (AffixLootEntry)WeightedRandom.m_216822_(ctx.m_230907_(), (List)resolved).get().m_146310_();
            final LootRarity selectedRarity2 = LootRarity.random(ctx.m_230907_(), ctx.m_78945_(), (this.rarityLimit == null) ? entry : this.rarityLimit);
            stack = LootController.createLootItem(entry.getStack().m_41777_(), selectedRarity2, ctx.m_230907_());
        }
        if (!stack.m_41619_()) {
            list.accept(stack);
        }
    }
    
    public LootPoolEntryType m_6751_() {
        return AffixLootPoolEntry.TYPE;
    }
    
    private AffixLootEntry unwrap(final DynamicHolder<AffixLootEntry> holder) {
        if (!holder.isBound()) {
            AdventureModule.LOGGER.error("An AffixLootPoolEntry failed to resolve the Affix Loot Entry {}!", (Object)holder.getId());
            return null;
        }
        return (AffixLootEntry)holder.get();
    }
    
    static {
        TYPE = new LootPoolEntryType((net.minecraft.world.level.storage.loot.Serializer)new Serializer());
    }
    
    public static class Serializer extends LootPoolSingletonContainer.Serializer<AffixLootPoolEntry>
    {
        protected AffixLootPoolEntry deserialize(final JsonObject obj, final JsonDeserializationContext context, final int weight, final int quality, final LootItemCondition[] lootConditions, final LootItemFunction[] lootFunctions) {
            RarityClamp.Simple rarity;
            if (obj.has("min_rarity") || obj.has("max_rarity")) {
                final DynamicHolder<LootRarity> minRarity = RarityRegistry.byLegacyId(GsonHelper.m_13906_(obj, "min_rarity"));
                final DynamicHolder<LootRarity> maxRarity = RarityRegistry.byLegacyId(GsonHelper.m_13906_(obj, "max_rarity"));
                rarity = new RarityClamp.Simple(minRarity, maxRarity);
            }
            else {
                rarity = null;
            }
            final List<String> entries = context.deserialize(GsonHelper.m_13832_(obj, "entries", new JsonArray()), new TypeToken<List<String>>() {}.getType());
            return new AffixLootPoolEntry(rarity, entries.stream().map((Function<? super Object, ? extends ResourceLocation>)ResourceLocation::new).toList(), weight, quality, lootConditions, lootFunctions);
        }
        
        public void serializeCustom(final JsonObject object, final AffixLootPoolEntry e, final JsonSerializationContext ctx) {
            if (e.rarityLimit != null) {
                object.addProperty("min_rarity", e.rarityLimit.min().getId().toString());
                object.addProperty("max_rarity", e.rarityLimit.max().getId().toString());
            }
            object.add("entries", ctx.serialize(e.entries));
            super.m_7219_(object, (LootPoolSingletonContainer)e, ctx);
        }
    }
}
